import pygame
import random
import os
import random

from scene import Scene

from wall import Wall
from gameOver import GameOver
from player import Player
from particles import *
from paintbucket import PaintBucket
from door import Door
from problems import ProblemsManager
from platform import Platform
from pygame.locals import *

################################################################################
# CLASS DEFINITION
################################################################################

class GameScene(Scene):
   
   def __init__(self,gameEngine):
      Scene.__init__(self, gameEngine)
      
      self.player = None
      self.objects = []
      self.paintBuckets = []
      self.doors = []
      self.doorPlacement = []
      self.levelDone = 0
      self.levelDoneTimer = 0.0
      self.levelDoneString = "temp text"
	  
      self.gameEngine = gameEngine
      
      self.spawn = (0,0) #spawn point of current level
      self.spawnTime = 0 #time until the player is respawned
	  
      self.problemsManager = ProblemsManager()      
      self.particleManager = ParticleManager(self)
      self.trailManager = TrailParticleManager(self)
      self.player = Player(self,200,250)
      pygame.font.init
      self.level = 1
      self.loadLevel()
	  
      
   
   #----------------------------------------------------------------------------
      
   def update(self):
   
      self.particleManager.update()
      self.trailManager.update(self.player.rect.x,self.player.rect.y,self.player.velX,self.player.velY)
	  
      if self.levelDone and self.levelDoneTimer <= 2.0:
         self.levelDoneTimer += .01
      elif self.levelDoneTimer > 2.0:
         self.levelDone = 0
         self.levelDoneTimer = 0.0
   
      if self.spawnTime == 0:
         self.player.update()
         self.wallCollision()
         self.problemCollision()
         
      #player needs to be respawned
      elif self.spawnTime == 1:
         self.spawnTime = 0
         self.player = Player(self,self.spawn[0],self.spawn[1])
      else:
         self.spawnTime-=1
         
   #----------------------------------------------------------------------------
   
   
   def problemCollision(self):
   
      keys = pygame.key.get_pressed()
      counter = 0
      for paintbucket in self.paintBuckets:
         if not paintbucket.hit: 
            if self.player.rect.colliderect(paintbucket.rect):
               paintbucket.hit = 1
               self.explosion(paintbucket.rect.x, paintbucket.rect.y, paintbucket.colour)
               #self.trail((20, 255, 0))
			   #add in player particles here
         else:
            counter += 1
      if(counter == 2):
            for door in self.doors:
               if self.player.rect.colliderect(door.rect):
                  #check to see if the player is trying to enter this door
                   if(keys[pygame.K_w]):
				       #if he wants to enter.. check to see if this is right door
                        self.levelDone = 1
                        if(door.realDoor):
					       #if it is right door..
                             self.levelDoneString = "That was the right door! Good Job!"
								#next level  
                             self.loadLevel()
                        else:
                             self.loadLevel()
								#restart map.. tell player he sucks
                             self.levelDoneString = "Wrong door! Try again!"
						
             
   def wallCollision(self):
     
	 #loops through objects/walls and tests the player character against them by  first seeing it collides.. then testing 8 points evenly spaced around the player's rectangle to see  which collision should happen.
	 #tests the point in the middle of the line on each side first.. then the corners... if either corner hits then that's the side that is mainly colliding so we adjust the players x or y accordingly.
      for object in self.objects:
         if self.player.rect.colliderect(object.rect):
			#test to see if it collides on the left side of the object
            if object.rect.collidepoint((self.player.rect.x + self.player.rect.width), self.player.rect.y + self.player.rect.height/2):
                if object.rect.collidepoint((self.player.rect.x + self.player.rect.width), self.player.rect.y) or object.rect.collidepoint((self.player.rect.x + self.player.rect.width), self.player.rect.y + self.player.rect.height):
                    self.player.rect.right = object.rect.left
			#test to see if it collides on the right side of the object
            elif object.rect.collidepoint(self.player.rect.x, self.player.rect.y + self.player.rect.height/2):
                if object.rect.collidepoint(self.player.rect.x, self.player.rect.y) or object.rect.collidepoint(self.player.rect.x , self.player.rect.y + self.player.rect.height):
                    self.player.rect.left = object.rect.right
				
			#test to see if it collides on the top side of the object	
            if object.rect.collidepoint(self.player.rect.x + self.player.rect.width/2, self.player.rect.y + self.player.rect.height):
                if object.rect.collidepoint(self.player.rect.x, self.player.rect.y + self.player.rect.height) or object.rect.collidepoint(self.player.rect.x + self.player.rect.width, self.player.rect.y + self.player.rect.height):
                    self.player.rect.bottom = object.rect.top
                    self.player.jumping = 0;
			#test to see if it collides on the bottom side of the object
            elif object.rect.collidepoint(self.player.rect.x + self.player.rect.width/2, self.player.rect.y):
                if object.rect.collidepoint(self.player.rect.x, self.player.rect.y) or object.rect.collidepoint(self.player.rect.x + self.player.rect.width, self.player.rect.y):
                    self.player.rect.top = object.rect.bottom
                    self.player.velY = 0
      
      
   def render(self):
      
      for obj in self.objects:
         obj.draw()
      for pBucket in self.paintBuckets:
         pBucket.draw()
      for door in self.doors:
         door.draw()
      self.particleManager.render()
      self.trailManager.render()
      
      if self.player is not None:
         self.player.draw()
         #self.drawHUD()
		 
      if self.levelDone:
        font = pygame.font.Font(None, 24)
        text = font.render(self.levelDoneString, 1, (255,255,255))
        self.gameEngine.screen.blit(text, (200,10))
         
   #----------------------------------------------------------------------------
   
   def drawHUD(self):
      
      gE = self.gameEngine
      
      #left arrow
      imgStr = os.path.join("hud",COLOUR.colourToWord(self.player.rColour1))
      img = gE.resourceManager.getImage(imgStr)
      gE.screen.blit(img,(0,464))
      
      #right arrow
      imgStr = os.path.join("hud",COLOUR.colourToWord(self.player.rColour2))
      img = gE.resourceManager.getImage(imgStr)
      img = pygame.transform.flip(img,True,False)
      gE.screen.blit(img,(16,464))
         
   #----------------------------------------------------------------------------
   
   def explosion(self,x,y,colour):
      numParticles = random.random()*100+100
      
      for i in range(0,int(numParticles)):
         
         xspeed = random.random()*5
         yspeed = random.random()*5
         
         if int(xspeed) is 0:
            yspeed+=1
         
         if random.random()<0.5:
            xspeed *= -1
         
         if random.random()<0.5:
            yspeed *= -1 
         
         particle = Particle(x,y,xspeed,yspeed,colour,random.random()*90+30)
         
         self.particleManager.queueAddParticle(particle)
   
   #----------------------------------------------------------------------------
   
   def trail(self,colour):
      numParticles = random.random()*100+100
      
      xspeed = 0
      yspeed = 0

      for i in range(0,int(numParticles)):
         
         xspeedRand = random.random()*2
         yspeedRand = random.random()*2
	  
         if self.player.velX < 0:
            xspeed -= xspeedRand
         else:
            xspeed += xspeedRand
		 
         if self.player.velY < 0:
            yspeed -= yspeedRand
         else:
            yspeed += yspeedRand
         
         particle = Particle(self.player.rect.x,self.player.rect.y,xspeed,yspeed,colour,random.random()*90+30)
         
         self.trailManager.queueAddParticle(particle)
   
   #----------------------------------------------------------------------------
      
   def loadLevel(self):
      try:
         f = open("data/levels/%03d"%self.level)
      except IOError:
         self.gameEngine.scene = GameOver(self.gameEngine)
         return
      
      del self.objects[:]
      del self.doorPlacement[:]
      del self.doors[:]
      del self.paintBuckets[:]
      x=y=0 
	  
      problem = self.problemsManager.get2_3Problem()
      qCounter = 0
      aCounter = 0
	  
      
      whileBool = 1
      randomBool = 1
	  
      randomNum = random.randrange(0,3)
      self.doorPlacement.append(randomNum)
	  
      for i in range(0,2): #loops through 0 , 1 ,2 and picks a random number to put on each of them
         while whileBool:# while loop starts
            randomNum = random.randrange(0,3)  #gets a random number
            for j in range(0, len(self.doorPlacement)): #check if its in the list already
                randomBool = 1  #sets the bool to true to say this is a good number
                if randomNum == self.doorPlacement[j]:
                    randomBool = 0  # if its a bad number.. then make teh bool false and break from the loop
                    break
					
			
            if randomBool:  #if it was a good number
                self.doorPlacement.append(randomNum) #add it to the list.. break the while
                whileBool = 0
         whileBool = 1  #set it so the while will go again
      
      while 1:
         row = f.readline()
         if not row:
            break
            
         for col in row:
		
            
            obj=None
            pBuckets=None
            exitDoors=None
            
            #S == start
            if col == "S":
				self.player = Player(self,x,y)
				self.spawn = (x,y)
               
            if col == "w":
                obj = Wall(self,x,y)
            if col == "q":
                obj = Platform(self,x,y)
            if col == "d":
                 if self.doorPlacement[aCounter] == problem.rightAnswer:
                     exitDoors = Door(self,x,y,problem.answers[self.doorPlacement[aCounter]].numerator,problem.answers[self.doorPlacement[aCounter]].denominator,1)
                 else:					 
                     exitDoors = Door(self,x,y,problem.answers[self.doorPlacement[aCounter]].numerator,problem.answers[self.doorPlacement[aCounter]].denominator,0)
                 aCounter += 1
            if col == "p":
                pBuckets = PaintBucket(self,x,y,problem.questions[qCounter].numerator,problem.questions[qCounter].denominator, problem.questions[qCounter].colour)
                qCounter += 1
            if obj is not None:
				self.objects.append(obj)
            if pBuckets is not None:
                self.paintBuckets.append(pBuckets)
            if exitDoors is not None:
                self.doors.append(exitDoors)
            x+=16
         y+=16
         x=0
