################################################################################
# particles.py
################################################################################
# Contains classes for the particle system.
################################################################################
# 09/09 Flembobs
################################################################################

import pygame
import random

################################################################################
# Particles
################################################################################

class Particle:
   
   def __init__(self,x,y,xspeed,yspeed,colour,lifespan):
      self.x = x
      self.y = y
      self.xspeed = xspeed
      self.yspeed = yspeed
      self.colour = colour
      self.lifespan = lifespan
	  
      self.startingX = x
      self.startingY = y
      self.startingXSpeed = xspeed
      self.startingYSpeed = yspeed
      self.startingLifespan = lifespan
      
   def draw(self,surface):
      pygame.draw.rect(surface,self.colour,pygame.Rect((self.x,self.y),(3,3)))
      
   def update(self):
      
      self.x+=self.xspeed
      self.y+=self.yspeed
      
      self.lifespan -= 1
	
   def makeNewVel(self, x, y):
     
	  
      xspeedRand = random.random()*2
      yspeedRand = random.random()*2
	  
      if x < 0:
         self.xspeed -= xspeedRand
      else:
         self.xspeed += xspeedRand
		 
      if y < 0:
         self.yspeed -= yspeedRand
      else:
         self.yspeed += yspeedRand
      
   

   
  
################################################################################
# Particle Manager
################################################################################
   
class ParticleManager:
   
   def __init__(self,gameScene):
      self.gameScene = gameScene
      self.particles = []
      self.particlesToAdd = []
      self.particlesToDel = []
      
   def update(self):
      
      for particle in self.particles:
         
         #reduce lifespan and remove the particle if it is zero
         if particle.lifespan <= 0:
            self.queueDelParticle(particle)
            continue
         particle.update()
      
      self.delQueued()
      self.addQueued()
   
   def render(self):
      for particle in self.particles:
         particle.draw(self.gameScene.gameEngine.screen)
   
   #############################################################################
   # Adding/Removing Particles
   #############################################################################
         
   def queueAddParticle(self,particle):
      self.particlesToAdd.append(particle)
      
   def queueDelParticle(self,particle):
      self.particlesToDel.append(particle)
      
   def __addParticle(self,particle):
      self.particles.append(particle)
      
   def __delParticle(self,particle):
      self.particles.remove(particle)
      
   def addQueued(self):
      for particle in self.particlesToAdd:
         self.__addParticle(particle)
         
      del self.particlesToAdd[:]
         
   def delQueued(self):
      for particle in self.particlesToDel:
         self.__delParticle(particle)
      
      del self.particlesToDel[:]
	  
	  

	  
class TrailParticleManager:
   
   def __init__(self,gameScene):
      self.gameScene = gameScene
      self.particles = []
      self.particlesToAdd = []
      self.particlesToDel = []
      
   def update(self, playerX, playerY, playerXVel, playerYVel):
      
      for particle in self.particles:
         
         #reduce lifespan and remove the particle if it is zero
         if particle.lifespan <= 0:
            self.queueDelParticle(particle)
            particle.lifespan = particle.startingLifespan
            particle.x = playerX
            particle.y = playerY
            particle.makeNewVel(playerXVel, playerYVel)
            self.queueAddParticle(particle)    
            continue
         particle.update()
      
      self.delQueued()
      self.addQueued()
   
   def render(self):
      for particle in self.particles:
         particle.draw(self.gameScene.gameEngine.screen)
   
   #############################################################################
   # Adding/Removing Particles
   #############################################################################
         
   def queueAddParticle(self,particle):
      self.particlesToAdd.append(particle)
      
   def queueDelParticle(self,particle):
      self.particlesToDel.append(particle)
      
   def __addParticle(self,particle):
      self.particles.append(particle)
      
   def __delParticle(self,particle):
      self.particles.remove(particle)
      
   def addQueued(self):
      for particle in self.particlesToAdd:
         self.__addParticle(particle)
         
      del self.particlesToAdd[:]
         
   def delQueued(self):
      for particle in self.particlesToDel:
         self.__delParticle(particle)
      
      del self.particlesToDel[:]
